"""MR Star light device commands."""
from .color import HSColor
from .const import COMMAND_PREFIX, COMMAND_SUFFIX, Command, Effect
from .int import split_uint16


def format_command(command: Command, args: bytes) -> bytes:
    """Formats command with prefix and suffix."""
    if len(args) == 0:
        raise ValueError("Command cannot be empty")
    return bytes([COMMAND_PREFIX, command.value, len(args), *args, COMMAND_SUFFIX])

def format_power_command(is_on: bool) -> bytes:
    """Formats power command."""
    return format_command(Command.SET_POWER, [(1 if is_on else 0)])

def format_brightness_command(brightness: float) -> bytes:
    """Formats brightness command."""
    if brightness < 0 or brightness > 1:
        raise ValueError("Brightness must be between 0 and 1")
    brightness_value = int(1024 * brightness)
    high_byte, low_byte = split_uint16(brightness_value)

    return format_command(Command.SET_BRIGHTNESS, [
        high_byte, low_byte, 0x00, 0x00, 0x00, 0x00
    ])

def format_color_command(color: HSColor) -> bytes:
    """Formats color command."""
    hue, sat = color
    hue_high, hue_low = split_uint16(hue)
    sat_high, sat_low = split_uint16(int(sat * 10))

    return format_command(Command.SET_COLOR, bytes([
        hue_high, hue_low, sat_high, sat_low, 0x00, 0x00
    ]))

def format_reverse_command(is_on: bool) -> bytes:
    """Formats reverse command."""
    return format_command(Command.SET_REVERSE, [(1 if is_on else 0)])

def format_speed_command(speed: float) -> bytes:
    """Formats speed command."""
    if speed < 0 or speed > 1:
        raise ValueError("Speed must be between 0 and 1")
    speed_value = int(100 * speed)
    return format_command(Command.SET_SPEED, [speed_value])

def format_effect_command(effect: Effect) -> bytes:
    """Formats effect command."""
    effect_high, effect_low = split_uint16(effect.value)
    return format_command(Command.SET_EFFECT, [effect_high, effect_low])

def format_length_command(led_count: int) -> bytes:
    """Formats length command."""
    if led_count < 8 or led_count > 300:
        raise ValueError("LED count must be between 8 and 300")
    count_high, count_low = split_uint16(led_count)
    return format_command(Command.SET_LENGTH, [count_high, count_low])



"""integer utils tests"""

import pytest

from mr_star_ble.int import split_uint16


def test_split_uint16():
    """Test split_uint16"""
    assert split_uint16(0x1234) == (0x12, 0x34)
    assert split_uint16(0x0078) == (0x00, 0x78)
    assert split_uint16(0x8000) == (0x80, 0x00)

    try:
        split_uint16(-1)
        pytest.fail(Exception("Expected ValueError"))
    except ValueError:
        assert True

    try:
        split_uint16(0x10000)
        pytest.fail(Exception("Expected ValueError"))
    except ValueError:
        assert True



"""Discover MR Star light device."""
import asyncio

from bleak import BleakScanner, BLEDevice

from .const import LIGHT_SERVICE


async def discover(timeout=10) -> list[BLEDevice]:
    """Discovers MR Star light device and returns the address."""
    devices: list[BLEDevice] = []

    def handle_discovery(device: BLEDevice, _):
        if device not in devices:
            devices.append(device)

    async with BleakScanner(handle_discovery, service_uuids=[LIGHT_SERVICE]) as _:
        await asyncio.sleep(timeout)

    return devices



"""tween utils tests"""
import pytest

from mr_star_ble.transition.frame import set_framerate
from mr_star_ble.transition.tween import HSColorTween, ValueTween


def test_forward_diff_tween():
    """Test forward diff tween between two values."""
    set_framerate(6)
    tween = ValueTween(0, 1, 1)
    assert next(tween) == 0.25
    assert next(tween) == 0.5
    assert next(tween) == 0.75
    assert next(tween) == 1
    with pytest.raises(StopIteration):
        next(tween)

    tween = ValueTween(1, 0, 1)
    assert next(tween) == 0.75
    assert next(tween) == 0.5
    assert next(tween) == 0.25
    assert next(tween) == 0
    with pytest.raises(StopIteration):
        next(tween)

def test_looped_diff_tween():
    """Test looped diff tween between two values."""
    set_framerate(4)
    hue_tween = ValueTween(0, 100, 1, loop_max=360)
    assert next(hue_tween) == 50
    assert next(hue_tween) == 100

    with pytest.raises(StopIteration):
        next(hue_tween)

    set_framerate(10)
    hue_tween = ValueTween(0, 350, 1, loop_max=360)
    assert next(hue_tween) == 358.75
    assert next(hue_tween) == 357.5
    assert next(hue_tween) == 356.25
    assert next(hue_tween) == 355.0
    assert next(hue_tween) == 353.75
    assert next(hue_tween) == 352.5
    assert next(hue_tween) == 351.25
    assert next(hue_tween) == 350

    with pytest.raises(StopIteration):
        next(hue_tween)

def test_hs_tween():
    """Test HS tween between two values."""
    set_framerate(6)
    tween = HSColorTween((0, 0), (100, 100), 1)
    assert next(tween) == (25, 25)
    assert next(tween) == (50, 50)
    assert next(tween) == (75, 75)
    assert next(tween) == (100, 100)

    with pytest.raises(StopIteration):
        next(tween)

    tween = HSColorTween((100, 100), (0, 0), 1)
    assert next(tween) == (75, 75)
    assert next(tween) == (50, 50)
    assert next(tween) == (25, 25)
    assert next(tween) == (0, 0)

    with pytest.raises(StopIteration):
        next(tween)

    tween = HSColorTween((0, 0), (350, 0), 1)
    assert next(tween) == (357, 0)
    assert next(tween) == (355, 0)
    assert next(tween) == (352, 0)
    assert next(tween) == (350, 0)

    with pytest.raises(StopIteration):
        next(tween)



"""integer utils"""

UINT16_MAX = 65535

# 0 - 65535
UInt16 = int

# 0 - 255
UInt8 = int

def split_uint16(value: UInt16) -> tuple[UInt8, UInt8]:
    """Splits uint16 value into low and high bytes."""
    if 0 > value or value > UINT16_MAX:
        raise ValueError(f"Value must be between 0 and {UINT16_MAX}, got {value}")
    low_byte = value & 0xFF
    high_byte = (value >> 8) & 0xFF
    return high_byte, low_byte



"""MR Star light animation utils"""


from mr_star_ble.api import MrStarAPI
from mr_star_ble.color import HSColor

from .frame import sleep_frame
from .tween import HSColorTween, ValueTween


async def animate_brightness(
        api: MrStarAPI,
        from_value: float,
        to_value: float,
        duration=1.0):
    """Sets the brightness of the device."""
    for value in ValueTween(from_value, to_value, duration):
        await api.set_brightness(value)
        await sleep_frame()

async def animate_power(
        api: MrStarAPI,
        is_on: bool,
        current_brightness: float,
        duration=1.2):
    """Animates lights power off."""
    if is_on:
        await api.set_brightness(0)
        await api.set_power(True)
        await animate_brightness(api, 0, current_brightness, duration)
    else:
        await animate_brightness(api, current_brightness, 0, duration)
        await api.set_power(False)
        await api.set_brightness(current_brightness)

async def animate_color(
        api: MrStarAPI,
        from_color: HSColor,
        to_color: HSColor,
        duration=1.0):
    """Sets the color of the device."""
    for color in HSColorTween(from_color, to_color, duration):
        await api.set_hs_color(color)
        await sleep_frame()



"""MR Star light device constants."""
from enum import Enum

# Device UUIDs
LIGHT_CHARACTERISTIC = "0000fff3-0000-1000-8000-00805f9b34fb"
LIGHT_SERVICE = "00002022-0000-1000-8000-00805f9b34fb"

# Command parts
COMMAND_PREFIX = 0xBC
COMMAND_SUFFIX = 0x55

class Command(Enum):
    """MR Star light device commands."""
    SET_POWER = 0x01
    SET_LENGTH = 0x03
    SET_COLOR = 0x04
    SET_BRIGHTNESS = 0x05
    SET_EFFECT = 0x06
    SET_REVERSE = 0x07
    SET_SPEED = 0x08

class Effect(Enum):
    """MR Star light device effects."""
    # Basic
    AUTOMATIC_LOOP = 1
    SYMPHONY = 2
    COLORFUL_ENERGY = 3
    COLORFUL_JUMPS = 4
    # RGB_JUMPS = 5
    # YBP_JUMPS = 6
    RAINBOW_STROBE = 7
    RAINBOW_GRADIENT = 10
    COLORFUL_FLUTTERING = 26
    RGB_FLUTTERING = 27
    YCP_FLUTTERING = 28
    COLORFUL_BRUSHING = 29
    RGB_COLOR_BRUSHING = 30
    YCP_COLOR_BRUSHING = 31
    COLOR_BRUSH_CLOSED_PULL = 32
    # RGB_CLOSED_PULL = 33
    # YCP_CLOSED_PULL = 34
    # Open & Close
    RAINBOW_OPEN_CLOSE = 35
    RGB_OPEN_CLOSE = 36
    YCP_OPEN_CLOSE = 37
    RED_OPEN_CLOSE = 38
    GREEN_OPEN_CLOSE = 39
    BLUE_OPEN_CLOSE = 40
    YELLOW_OPEN_CLOSE = 41
    CYAN_OPEN_CLOSE = 42
    PURPLE_OPEN_CLOSE = 43
    WHITE_OPEN_CLOSE = 44
    # Transition
    RAINBOW_LIGHT_DARK_TRANSITION = 45
    RGB_LIGHT_DARK_TRANSITION = 46
    VGY_LIGHT_DARK_TRANSITION = 47
    RED_LIGHT_DARK_TRANSITION = 48
    GREEN_LIGHT_DARK_TRANSITION = 49
    BLUE_LIGHT_DARK_TRANSITION = 50
    CYAN_LIGHT_DARK_TRANSITION = 51
    YELLOW_LIGHT_DARK_TRANSITION = 52
    PURPLE_LIGHT_DARK_TRANSITION = 53
    WHITE_LIGHT_DARK_TRANSITION = 54
    # Flowing water
    RAINBOW_FLOWING_WATER = 55
    RGB_RUNNING_WATER = 56
    VY_RUNNING_WATER = 57
    RG_RUNNING_WATER = 58
    GB_RUNNING_WATER = 59
    YB_RUNNING_WATER = 60
    YC_RUNNING_WATER = 61
    GP_RUNNING_WATER = 62
    # BW_RUNNING_WATER = 63
    # Trailing
    RAINBOW_TRAILING = 76
    RED_TRAILING = 77
    GREEN_TRAILING = 78
    BLUE_TRAILING = 79
    YELLOW_TRAILING = 80
    CYAN_TRAILING = 81
    PURPLE_TRAILING = 82
    WHITE_TRAILING = 83
    # Running
    RED_RUNNING = 84
    GREEN_RUNNING = 85
    BLUE_RUNNING = 86
    YELLOW_RUNNING = 87
    CYAN_RUNNING = 88
    PURPLE_RUNNING = 89
    YPB_STROBE = 9
    WHITE_RUNNING = 90
    RAINBOW_RUNNING = 91
    BGR_RUNNING = 92
    PCY_RUNNING = 93
    BPCY_RUNNING = 94
    BGCY_RUNNING = 95



"""Example of automatically discovering and connecting to a MR Star light device."""
import asyncio

from bleak import BleakClient

from mr_star_ble import discover


async def main():
    """Auto discover and connect to a MR Star light device."""
    print("Searching...")
    devices = await discover()
    if len(devices) == 0:
        print("No devices found")
        return
    print(f"Devices found: {len(devices)}")
    for device in devices:
        print(f"Connecting to {device.address}...")
        client = BleakClient(device)
        await client.connect()
        print(f"Connected to {device.address}")
        await client.disconnect()

asyncio.run(main())



"""Integer diff utils."""

def forward_diff(from_value: float, to_value: float) -> float:
    """Returns forward-front difference between two values."""
    return to_value - from_value

def looped_diff(from_value: float, to_value: float, max_value: float) -> float:
    """Returns shortest difference between two values."""
    forward = forward_diff(from_value, to_value)
    backward = (max_value - max(from_value, to_value) + min(from_value, to_value))
    if backward < abs(forward):
        if from_value < to_value:
            backward *= -1
        return backward
    return forward



"""integer utils tests"""
from mr_star_ble.transition.diff import forward_diff, looped_diff

HUE_MAX = 360

def test_forward_diff():
    """Test split_uint16"""
    assert forward_diff(0, 0) == 0
    assert forward_diff(0, 1) == 1
    assert forward_diff(255, 10) == -245
    assert forward_diff(10, 255) == 245

def test_looped_diff():
    """Test split_uint16"""
    assert looped_diff(0, 360, HUE_MAX) == 0
    assert looped_diff(0, 1, HUE_MAX) == 1
    assert looped_diff(359, 0, HUE_MAX) == 1
    assert looped_diff(0, 359, HUE_MAX) == -1
    assert looped_diff(30, 350, HUE_MAX) == -40
    assert looped_diff(350, 30, HUE_MAX) == 40
    assert looped_diff(30, 30, HUE_MAX) == 0



"""MR Star light device."""

from bleak import BleakClient

from .color import HSColor, RGBColor, rgb_to_hs
from .commands import (
    format_brightness_command,
    format_color_command,
    format_command,
    format_effect_command,
    format_length_command,
    format_power_command,
    format_reverse_command,
    format_speed_command,
)
from .const import LIGHT_CHARACTERISTIC, Effect


class MrStarAPI:
    """Represents a MR Star light device API."""
    _client: BleakClient

    def __init__(self, client: BleakClient):
        self._client = client

    @property
    def is_connected(self) -> bool:
        """Check connection status between this client and the GATT server."""
        return self._client.is_connected

    async def set_power(self, is_on: bool):
        """Sets the power state of the device."""
        await self.write(format_power_command(is_on))

    async def set_length(self, length: int):
        """Sets the power state of the device."""
        await self.write(format_length_command(length))

    async def set_effect(self, effect: Effect):
        """Sets the effect of the device."""
        await self.write(format_effect_command(effect))

    async def set_reverse(self, is_on: bool):
        """Sets the power state of the device."""
        await self.write(format_reverse_command(is_on))

    async def set_speed(self, speed: float):
        """Sets the power state of the device."""
        await self.write(format_speed_command(speed))

    async def set_brightness(self, brightness: float):
        """Sets the brightness of the device."""
        await self.write(format_brightness_command(brightness))

    async def set_hs_color(self, color: HSColor):
        """Sets the color of the device."""
        await self.write(format_color_command(color))

    async def set_rgb_color(self, color: RGBColor):
        """Sets the color of the device."""
        await self.set_hs_color(rgb_to_hs(color))

    async def write_command(self, command: int, argument: bytes):
        """Writes a payload to the device."""
        await self.write(format_command(command, argument))

    async def write(self, payload: bytes):
        """Writes a raw payload to the device."""
        if not self.is_connected:
            raise RuntimeError("Device is not connected")
        await self._client.write_gatt_char(
            LIGHT_CHARACTERISTIC, payload, response=False)



"""Animation frame utils."""

import asyncio

_FRAMERATE = 30
_FRAME_TIME = 1 / _FRAMERATE

def set_framerate(framerate: int):
    """Sets global framerate for all animation iterators."""
    global _FRAMERATE # pylint: disable=global-statement
    global _FRAME_TIME # pylint: disable=global-statement
    _FRAMERATE = framerate
    _FRAME_TIME = 1 / _FRAMERATE

def get_frame_time() -> float:
    """Returns global frame time for all animation iterators."""
    return _FRAME_TIME

async def sleep_frame():
    """Sleeps the frame time."""
    await asyncio.sleep(_FRAME_TIME)



"""Tween animation."""
from functools import partial
from typing import Callable

from mr_star_ble.color import HSColor

from .diff import forward_diff, looped_diff
from .frame import get_frame_time

HUE_MAX = 360

Differ = Callable[[float, float], float]

class ValueTween:
    """Tween iterator between two values."""
    _from_value: float
    _to_value: float
    _step: float
    _duration: float
    _frame: int
    _frame_count: int
    _diff: Differ
    _loop_max: float

    @property
    def from_value(self) -> float:
        """Returns from value."""
        return self._from_value

    @property
    def to_value(self) -> float:
        """Returns to value."""
        return self._to_value

    def __init__(
            self,
            from_value: float,
            to_value: float,
            duration: float,
            loop_max: float | None = None):
        self._from_value = from_value
        self._to_value = to_value
        self._duration = duration
        if loop_max is None:
            self._diff = forward_diff
        else:
            self._diff = partial(looped_diff, max_value=loop_max)

        self._loop_max = loop_max
        self.__iter__()

    def __iter__(self):
        delta = self._diff(self._from_value, self._to_value)
        # Skip start and end frames
        self._frame_count = int(self._duration / get_frame_time()) - 2
        self._step = delta / self._frame_count
        self._frame = 0
        return self

    def __next__(self):
        if self._frame == self._frame_count:
            raise StopIteration
        self._frame += 1
        value = self._from_value + (self._step * self._frame)
        if self._loop_max is None:
            return value
        if value <= 0:
            return self._loop_max + value
        elif value >= self._loop_max:
            return value - self._loop_max
        return value

class HSColorTween:
    """Tween iterator between two HS colors."""
    _from: HSColor
    _to: HSColor

    def __init__(
            self,
            from_value: HSColor,
            to_value: HSColor,
            duration: float):
        hue_from, value_from = from_value
        hue_to, value_to = to_value

        self._hue = ValueTween(
            hue_from,
            hue_to,
            duration,
            loop_max=HUE_MAX)
        self._value = ValueTween(
            value_from,
            value_to,
            duration)

    def __iter__(self):
        self._hue.__iter__()
        self._value.__iter__()
        return self

    def __next__(self):
        hue = self._hue.__next__()
        value = self._value.__next__()
        if hue == HUE_MAX:
            hue = 0
        return int(hue), value



"""Enum block tests"""
import pytest

from mr_star_ble.commands import (
    format_brightness_command,
    format_color_command,
    format_command,
    format_effect_command,
    format_length_command,
    format_power_command,
    format_reverse_command,
    format_speed_command,
)
from mr_star_ble.const import Command, Effect


def test_format_command():
    """Test base command formatting"""
    assert format_command(Command.SET_POWER, [0x02, 0x03]) == bytes([
        0xBC, 0x01, 0x02, 0x02, 0x03, 0x55])
    try:
        format_command(Command.SET_POWER, [])
        pytest.fail(Exception("Expected ValueError"))
    except ValueError:
        assert True

def test_format_power_command():
    """Test power command formatting"""
    assert format_power_command(True) == bytes([
        0xBC, 0x01, 0x01, 0x01, 0x55])
    assert format_power_command(False) == bytes([
        0xBC, 0x01, 0x01, 0x00, 0x55])

def test_format_brightness_command():
    """Test brightness command formatting"""
    assert format_brightness_command(0) == bytes([
        0xBC, 0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55])
    assert format_brightness_command(1) == bytes([
        0xBC, 0x05, 0x06, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55])
    assert format_brightness_command(0.5) == bytes([
        0xBC, 0x05, 0x06, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55])

    try:
        format_brightness_command(2)
        pytest.fail(Exception("Expected ValueError"))
    except ValueError:
        assert True
    try:
        format_brightness_command(-1)
        pytest.fail(Exception("Expected ValueError"))
    except ValueError:
        assert True


def test_format_color_command():
    """Test color command formatting"""
    assert format_color_command((0, 100)) == bytes([
        0xBC, 0x04, 0x06, 0x00, 0x00, 0x03, 0xe8, 0x00, 0x00, 0x55])
    assert format_color_command((120, 100)) == bytes([
        0xBC, 0x04, 0x06, 0x00, 0x78, 0x03, 0xE8, 0x00, 0x00, 0x55])
    assert format_color_command((240, 100)) == bytes([
        0xBC, 0x04, 0x06, 0x00, 0xF0, 0x03, 0xE8, 0x00, 0x00, 0x55])

def test_format_reverse_command():
    """Test reverse command formatting"""
    assert format_reverse_command(True) == bytes([
        0xBC, 0x07, 0x01, 0x01, 0x55])
    assert format_reverse_command(False) == bytes([
        0xBC, 0x07, 0x01, 0x00, 0x55])

def test_format_speed_command():
    """Test speed command formatting"""
    assert format_speed_command(0.01) == bytes([
        0xBC, 0x08, 0x01, 0x01, 0x55])
    assert format_speed_command(0.5) == bytes([
        0xBC, 0x08, 0x01, 0x32, 0x55])
    assert format_speed_command(1) == bytes([
        0xBC, 0x08, 0x01, 0x64, 0x55])
    try:
        format_speed_command(2)
        pytest.fail(Exception("Expected ValueError"))
    except ValueError:
        assert True
    try:
        format_speed_command(-1)
        pytest.fail(Exception("Expected ValueError"))
    except ValueError:
        assert True

def test_format_effect_command():
    """Test effect command formatting"""
    assert format_effect_command(Effect.AUTOMATIC_LOOP) == bytes([
        0xBC, 0x06, 0x02, 0x00, 0x01, 0x55])
    assert format_effect_command(Effect.SYMPHONY) == bytes([
        0xBC, 0x06, 0x02, 0x00, 0x02, 0x55])
    assert format_effect_command(Effect.PURPLE_OPEN_CLOSE) == bytes([
        0xBC, 0x06, 0x02, 0x00, 0x2B, 0x55])

def test_format_length_command():
    """Test length command formatting"""
    assert format_length_command(8) == bytes([
        0xBC, 0x03, 0x02, 0x00, 0x08, 0x55])
    assert format_length_command(255) == bytes([
        0xBC, 0x03, 0x02, 0x00, 0xFF, 0x55])
    assert format_length_command(300) == bytes([
        0xBC, 0x03, 0x02, 0x01, 0x2C, 0x55])
    try:
        format_length_command(7)
        pytest.fail(Exception("Expected ValueError"))
    except ValueError:
        assert True
    try:
        format_length_command(301)
        pytest.fail(Exception("Expected ValueError"))
    except ValueError:
        assert True



"""Example of updating the state of a MR Star light device."""
import asyncio

from bleak import BleakClient

from mr_star_ble import MrStarAPI, discover
from mr_star_ble.transition import animate_color, animate_power, set_framerate

ADDRESS = "DF821C47-03A6-D4C5-D545-B7D3EE0B3172"

async def main():
    """Updates the state of a MR Star light device."""
    dev = await discover()

    client = BleakClient(dev)
    print("Connecting...")
    await client.connect()
    api = MrStarAPI(client)
    # hue_range = transition.ValueRange(0, 360)
    # print(hue_range.shortest_diff(360, 20))
    set_framerate(25)
    await api.set_rgb_color((255, 0, 0))
    await animate_color(api, (0, 100), (120, 100), duration=1)
    await asyncio.sleep(3)
    await animate_power(api, False, 1.0, duration=1.2)
    await asyncio.sleep(3)
    await animate_power(api, True, 1.0, duration=1.2)
    await asyncio.sleep(3)
    await animate_color(api, (120, 100.0), (0, 100.0), duration=1)

    #     await transition.sleep_frame()
    # await client.disconnect()

    # api = MrStarAPI(client)
    # animator = MrStarAnimator(api, framerate=30)
    # await animator.set_brightness(1, duration=1.2)
    # await asyncio.sleep(2)
    # await animator.set_brightness(0, duration=1.2)
    # await animator.set_brightness(1, duration=2)
    # await animator.set_brightness(1, duration=1)
    # await animator.set_brightness(0.5, duration=1)
        # await light.set_rgb_color((255, 0, 0))

asyncio.run(main())



"""MR Star device color utils."""
from colorsys import rgb_to_hsv

RGBColor = tuple[int, int, int]
HSColor = tuple[int, float] # Hue 0-360, Saturation 0-100

def rgb_to_hs(rgb: RGBColor) -> HSColor:
    """Convert RGB color to hue and saturation."""
    red, green, blue = rgb
    hue, saturation, _ = rgb_to_hsv(
        float(red / 255),
        float(green / 255),
        float(blue / 255)
    )
    return int(hue * 360), saturation * 100



"""Test color conversions"""
from mr_star_ble.color import rgb_to_hs


def test_rgb_to_hs():
    """Test RGB to HS conversion"""
    assert rgb_to_hs((255, 255, 255)) == (0, 0)
    assert rgb_to_hs((255, 0, 0)) == (0, 100)
    assert rgb_to_hs((0, 255, 0)) == (120, 100)
    assert rgb_to_hs((0, 0, 255)) == (240, 100)
    assert rgb_to_hs((128, 128, 0)) == (60, 100)
    assert rgb_to_hs((0, 0, 0)) == (0, 0)



"""Test color conversions"""

import asyncio

import pytest

from mr_star_ble import Effect, MrStarDevice, discover


@pytest.mark.asyncio
async def test_power():
    """Test RGB to HS conversion"""
    ble_device = await discover()
    device = MrStarDevice(ble_device)
    await device.connect()
    async with device as light:
        await light.set_power(False)
        await asyncio.sleep(0.2)
        await light.set_power(True)
        await asyncio.sleep(0.5)
        await light.set_power(False)
        await asyncio.sleep(0.5)
        await light.set_power(True)
    await device.disconnect()

@pytest.mark.asyncio
async def test_brightness():
    """Test RGB to HS conversion"""
    ble_device = await discover()
    device = MrStarDevice(ble_device)
    await device.connect()
    async with device as light:
        brightness = 0.1
        while brightness <= 1:
            await light.set_brightness(brightness)
            await asyncio.sleep(0.2)
            brightness += 0.1
    await device.disconnect()

@pytest.mark.asyncio
async def test_color():
    """Test RGB to HS conversion"""
    ble_device = await discover()
    device = MrStarDevice(ble_device)
    await device.connect()
    async with device as light:
        await light.set_rgb_color((255, 0, 0))
        await asyncio.sleep(0.5)
        await light.set_rgb_color((0, 255, 0))
        await asyncio.sleep(0.5)
        await light.set_rgb_color((0, 0, 255))
        await asyncio.sleep(0.5)
        await light.set_rgb_color((255, 255, 0))
        await asyncio.sleep(0.5)
        await light.set_rgb_color((255, 0, 255))
        await asyncio.sleep(0.5)
        await light.set_rgb_color((0, 255, 255))
        await asyncio.sleep(0.5)
        await light.set_rgb_color((255, 255, 255))
    await device.disconnect()

# @pytest.mark.asyncio
# async def test_effect():
#     """Test RGB to HS conversion"""
#     ble_device = await discover()
#     device = MrStarDevice(ble_device)
#     await device.connect()
#     async with device as light:
#         for effect in Effect:
#             await light.set_effect(effect)
#             await asyncio.sleep(0.5)
#     await device.disconnect()

@pytest.mark.asyncio
async def test_speed():
    """Test RGB to HS conversion"""
    ble_device = await discover()
    device = MrStarDevice(ble_device)
    await device.connect()
    async with device as light:
        await light.set_effect(Effect.SYMPHONY)
        speed = 0.1
        while speed <= 1:
            await light.set_speed(speed)
            await asyncio.sleep(0.2)
            speed += 0.1
    await device.disconnect()

@pytest.mark.asyncio
async def test_reverse():
    """Test RGB to HS conversion"""
    ble_device = await discover()
    device = MrStarDevice(ble_device)
    await device.connect()
    async with device as light:
        await light.set_reverse(False)
        await light.set_effect(Effect.YELLOW_OPEN_CLOSE)
        await asyncio.sleep(0.5)
        await light.set_reverse(True)
        await asyncio.sleep(0.5)
        await light.set_reverse(False)
    await device.disconnect()
